from oda_wd_client.service.financial_management.api import FinancialManagement
from oda_wd_client.service.human_resources.api import HumanResources
from oda_wd_client.service.resource_management.api import ResourceManagement
from oda_wd_client.service.staffing.api import Staffing

__all__ = ["FinancialManagement", "HumanResources", "ResourceManagement", "Staffing"]
